use crate::config::Config;
use maud::{html, PreEscaped, DOCTYPE};
use rocket::request::{FromRequest, Outcome, Request};

pub enum RequestClient {
    Browser,
    Other,
}

#[rocket::async_trait]
impl<'r> FromRequest<'r> for RequestClient {
    type Error = ();

    async fn from_request(req: &'r Request<'_>) -> Outcome<Self, Self::Error> {
        if is_browser(req) {
            Outcome::Success(Self::Browser)
        } else {
            Outcome::Success(Self::Other)
        }
    }
}

#[must_use]
pub fn is_browser(req: &Request) -> bool {
    let ua = req
        .headers()
        .get("usesr-agent")
        .next()
        .unwrap()
        .to_lowercase();
    ua.contains("chrome") || ua.contains("safari") || ua.contains("firefox")
}

pub async fn build_site(
    content: String,
    title: &str,
    disable_color: bool,
    shadow: bool,
    config: &Config,
) -> String {
    let mut c_class = "bg-dark text-white justify-content-center text-center".to_string();
    let mut c_style = String::new();
    let mut g_style = "a {text-decoration: none; font-weight: bold; color: white}".to_string();
    if !disable_color {
        if let (Some(fg), Some(bg)) = (config.fg_color(), config.bg_color()) {
            c_class = "justify-content-center text-center".to_string();
            c_style = format!("background: {bg}; color: {fg};");
            g_style = format!("a {{text-decoration: none; font-weight: bold; color: {fg}}}");
        }
    }

    if std::path::Path::new("./config/wall.avif").exists() {
        c_style.push_str("background-image: url('assets/wall');background-size:cover;");
    }

    if shadow {
        c_style.push_str("text-shadow: 1px 1px 3px black;");
    }

    let body = html! {
            body style=(c_style) class=(c_class) {
                style { (g_style) };
                (PreEscaped(content))
            };
    };

    build_site_from_body(title, &body.into_string())
}

pub fn build_site_from_body(title: &str, body: &str) -> String {
    html! {
        (DOCTYPE)
        html {
            head {
                meta charset="UTF-8";
                title {
                    (title)
                };
                link rel="icon" href="/assets/me";
                meta name="viewport" content="width=device-width, initial-scale=1.0";
                link href="/bootstrap.min.css" rel="stylesheet";
                        link href="/bootstrap-icons.css" rel="stylesheet";
                        script src="/bootstrap.bundle.min.js" {};
                        link href="/style/global.css" rel="stylesheet";
                        link href="/style/site.css" rel="stylesheet";
                        link href="/style/common.css" rel="stylesheet";
            };
            (PreEscaped(body))
        };
    }
    .into_string()
}
